/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/*
 * test of various gtk stuff.
 *
 * Currently: try to bring up a window that just fits on the screen
 * vertically, but is half the width of the screen.
 *
 * To build as a standalone demo:
gcc `pkg-config --cflags --libs gtk+-2.0` -o gtktest -DSTANDALONE=1 gtktest.c
 */

#include <gtk/gtk.h>

GtkWidget *gWin = 0;
int gMonitorWidth, gMonitorHeight;
static GtkWidget *sDrawingArea = 0;

static gint HandleExpose(GtkWidget* widget, GdkEventExpose* event)
{
    gint winwidth, winheight;
    gint dawidth, daheight;
    gint framewidth, frameheight;
    GdkRectangle rect;
    static int width = 0, height = 0;

    gdk_drawable_get_size(widget->window, &winwidth, &winheight);
    gdk_drawable_get_size(sDrawingArea->window, &dawidth, &daheight);
    gdk_window_get_frame_extents(gWin->window, &rect);
    framewidth = rect.width - winwidth;
    frameheight = rect.height - winheight;

    printf("HandleExpose: area %dx%d +%d+%d in window %dx%d\n",
           event->area.width, event->area.height,
           event->area.x, event->area.y,
           winwidth, winheight);
    printf("  with frame %dx%d (da is %dx%d)\n",
           framewidth, frameheight, dawidth, daheight);

    if (winwidth != width || winheight != height)
    {
        printf("Window size is wrong -- changing it\n");
        width = gMonitorWidth/2;
        height = winheight - frameheight;
        printf("HandleExpose: resizing to %dx%d\n", width, height);
        gdk_window_unmaximize(gWin->window);
        gtk_drawing_area_size(GTK_DRAWING_AREA(sDrawingArea), width, height);
        gtk_window_resize(GTK_WINDOW(gWin), width, height);
    }
    else {
        width = winwidth;
        height = winheight;
    }
}

int main(int argc, char** argv)
{
    gint sFrameWidth = -1;
    gint sFrameHeight = -1;
    static GdkColor sBlack;
    gint winwidth, winheight;
    gint dawidth, daheight;
    GdkRectangle rect;

    gtk_init(&argc, &argv);
    gMonitorWidth = gdk_screen_width();
    gMonitorHeight = gdk_screen_height();

    gWin = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_wmclass(GTK_WINDOW(gWin), "pho", "Pho");

    sDrawingArea = gtk_drawing_area_new();
    gtk_container_add(GTK_CONTAINER(gWin), sDrawingArea);
    gtk_widget_show(sDrawingArea);

    /*gtk_drawing_area_size(GTK_DRAWING_AREA(sDrawingArea),
                          gMonitorWidth, gMonitorHeight);
    /*gtk_window_unfullscreen(GTK_WINDOW(gWin));*/

    gtk_signal_connect(GTK_OBJECT(sDrawingArea), "expose_event",
                       (GtkSignalFunc)HandleExpose, 0);
    gtk_signal_connect(GTK_OBJECT(gWin), "delete_event",
                       (GtkSignalFunc)gtk_main_quit, 0);

    gtk_widget_show(gWin);

    /* GTK can only maximize in both directions at once, not in one.
     * And maximize doesn't work before show() (else prints Gdk-CRITICAL err).
     * And we can't get the maximized size until the expose handler,
     * by which time the user has seen the whole screen flash. Not good.
    gdk_window_maximize(gWin->window);
     */

    /* Now resize horizontally.
     * But this doesn't work here -- a resize before the Expose
     * is ignored, and we can't get the frame size here anyway.
    gdk_drawable_get_size(gWin->window, &winwidth, &winheight);
    printf("Got size %dx%d; trying to resize to %dx%d\n",
           winwidth, winheight, gMonitorWidth/3, winheight);

    gdk_window_get_frame_extents(gWin->window, &rect);
    printf("frame size is %dx%d\n", rect.width-winwidth, rect.height-winheight);

    gdk_window_unmaximize(gWin->window);
    gtk_drawing_area_size(GTK_DRAWING_AREA(sDrawingArea),
                          gMonitorWidth/3, winheight);
    gtk_window_resize(GTK_WINDOW(gWin), gMonitorWidth/3, winheight);

    gdk_drawable_get_size(gWin->window, &winwidth, &winheight);
    gdk_drawable_get_size(sDrawingArea->window, &dawidth, &daheight);
    printf("After first resize, win is %dx%d and da is %dx%d\n",
           winwidth, winheight, dawidth, daheight);
     */

    gtk_main();
}
